/* GTK based ISDN monitor : inspired by the old isdnbutton program 
 * written in 1996 by M. Gutschke.
 * 
 * Kees Lemmens, Dec. 2001
 * Email: C.W.J.Lemmens@its.tudelft.nl
 */

#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "interface.h"
#include "main.h"

#include "isdnred.xpm"
#include "isdnyellow.xpm"
#include "isdngreen.xpm"
#include "isdnblue.xpm"

static GtkWidget *butbox1, *label1, *pixmap1, *button1, *window1;

void appClicked (GtkWidget *widget, gpointer data)
{
   /* fprintf(stderr,"Button clicked\n"); */

   switch(settings.state)
   {
    case 0 :  system(settings.netup   );  break;
    case 1 :  system(settings.netdown );  break;
    case 2 :
    case 3 :  system(settings.hangup );   break;
    default: fprintf(stderr,"Unkown state %d\n",
		     settings.state);   break;
   }
}

void switchStateIcon(int state)
{
   gtk_container_remove( GTK_CONTAINER(butbox1), pixmap1 );
   
   switch(state)
   {
    case 0 : pixmap1 = createPixmap(window1, isdnred);    break;
    case 1 : pixmap1 = createPixmap(window1, isdnyellow); break;
    case 2 : pixmap1 = createPixmap(window1, isdngreen);  break;
    case 3 : pixmap1 = createPixmap(window1, isdnblue);   break;
    default: fprintf(stderr,"Unkown state %d\n",state);   break;
   }
   
   gtk_container_add( GTK_CONTAINER(butbox1), pixmap1 );
}

GtkWidget* createPixmap(GtkWidget *widget, char *pixmap_xpm[])
{
   GtkWidget *pixmap;
   GdkColormap *colormap;
   GdkPixmap *gdkpixmap;
   GdkBitmap *mask;
   GtkStyle *style;
   
   /* now for the pixmap from gdk */
   colormap = gtk_widget_get_colormap (widget);
   style = gtk_widget_get_style(widget );
   gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d
     (NULL, colormap, &mask, NULL, (gchar **)pixmap_xpm );

   if (gdkpixmap == NULL)
     fprintf(stderr,"Couldn't create pixmap.");
   
   /* make a pixmap widget to contain the pixmap */
   pixmap = gtk_pixmap_new( gdkpixmap, mask );
   gtk_widget_show( pixmap );
   
   gdk_pixmap_unref (gdkpixmap);
   gdk_bitmap_unref (mask);
   return pixmap;
}

GtkWidget* createLabel(char *text)
{
   GtkWidget *label = gtk_label_new (text);
   
   gtk_widget_show(label);
   
   return label;
}

GtkWidget* createLabelledButton(void)
{
   GtkWidget *button = gtk_button_new();
   
   pixmap1 = createPixmap(window1, isdnred);
   label1  = createLabel(settings.label);
   
   /* Create a box and pack the pixmap and label into it */
   butbox1 = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (butbox1), 0);

   gtk_box_pack_start (GTK_BOX (butbox1), pixmap1, FALSE, FALSE, 3);
   gtk_box_pack_end   (GTK_BOX (butbox1), label1  ,FALSE, FALSE, 3);
   gtk_widget_show( butbox1 );
   
   gtk_container_set_border_width( GTK_CONTAINER (button), 0 );
   gtk_container_add( GTK_CONTAINER(button), butbox1 );

   gtk_widget_show( button );
   return button;
}

GtkWidget* createTopwindow (void)
{
   window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
   gtk_window_set_title (GTK_WINDOW (window1), "window1");
   gtk_container_set_border_width( GTK_CONTAINER (window1), 0 );
   
   button1 = createLabelledButton();
   gtk_container_add( GTK_CONTAINER(window1), button1 );
   
   gtk_signal_connect( GTK_OBJECT(button1), "clicked",
                        GTK_SIGNAL_FUNC(appClicked), NULL );

   return window1;
}
