// Get ISDN state : hacked together by Kees Lemmens, Dec 2001 

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <syslog.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include <linux/isdn.h>

enum state
{
   isRed, isYellow, isGreen, isBlue
};

// red    : no connection, no available device(s)
// yellow : no connection, available ippp device(s)
// green  : connection established, incoming
// blue   : connection established, outgoing

static enum state teststate()
{
#define BUFSZ 4096
   char buf[BUFSZ],temp[BUFSZ],*ptr;
   enum state state;
   FILE *fp;
   int i, num;
   
   state = isRed;
   
   /* Check if any isdn network device is configured (visible in ifconfig) */
   if ((fp = fopen("/proc/net/route","r")) != NULL)
   {
      while (!feof(fp) && fgets(buf,sizeof(buf),fp))
      {
	 if ((ptr = strstr(buf,"ippp")) != NULL)
	 {
	    state = isYellow;
	    break;
	 }
      }
      
      fclose(fp);
   }
   /* Check if any isdn network device is active (connected) */
   if (state == isYellow && (fp = fopen("/dev/isdninfo","r")) != NULL)
   {
      while(!feof(fp) && fgets(buf,sizeof(buf),fp))
	if ((ptr = strstr(buf,"usage")) != NULL)
	  break;
      fclose(fp);
      ptr += 7;               // skip tag + colon + space
      for (i=0; i<ISDN_MAX_CHANNELS; i++)
      {
	 sscanf(ptr,"%d",&num);
	 sscanf(ptr,"%s",temp);
	 ptr += strlen(temp) + 1;
	 
	 if ((num&ISDN_USAGE_MASK) == ISDN_USAGE_NET)
	 {
	    if (num&ISDN_USAGE_OUTGOING)
	      state = isGreen;
	    else
	      state = isBlue;
	 }
      }
   }
   
   return state;
}

static int initDaemon(void)
{
   pid_t      pid;
   int        fd,stat;
   
   /* Put process into background */
   if ((pid = fork()) < 0) {
      syslog(LOG_ERR,"Fork error: %s\n",strerror(errno));
      return(-1); }
   else if (pid != 0) {
      /* reap exit code to avoid zombies */
      while (waitpid(pid,&stat,0) < 0 && errno == EINTR);
      return(pid); }
   /* Get rid of parent process, so we will be adopted from init */
   exit(0);
   
   /* Now we are a true daemon process; make sure that all of the
    process environment is configured properly */
   setsid();
   chdir("/");
   umask(0);
   for (fd = sysconf(_SC_OPEN_MAX); fd--; )
     close(fd);
   openlog("isdnbutton",LOG_PID,LOG_USER);
   return(0);
}

int main (int argc, char *argv[])
{
   enum state state;
   
   while(1)
   {
      state = teststate();
      sleep(1);
      fprintf(stderr,"state : %d\n",state);
   }
   exit(0);
}

